@@ -45,7 +45,6 @@ ID oj_hash_key_id;
 ID oj_hash_set_id;
 ID oj_hash_start_id;
 ID oj_iconv_id;
-ID oj_instance_variables_id;
 ID oj_json_create_id;
 ID oj_length_id;
 ID oj_new_id;
@@ -137,6 +136,7 @@ static VALUE rails_sym;
 static VALUE raise_sym;
 static VALUE ruby_sym;
 static VALUE sec_prec_sym;
+static VALUE slash_sym;
 static VALUE strict_sym;
 static VALUE symbol_keys_sym;
 static VALUE time_format_sym;
@@ -153,6 +153,7 @@ static VALUE xmlschema_sym;
 static VALUE xss_safe_sym;
 
 rb_encoding *oj_utf8_encoding = 0;
+int oj_utf8_encoding_index = 0;
 
 #ifdef HAVE_PTHREAD_MUTEX_INIT
 pthread_mutex_t oj_cache_mutex;
@@ -404,6 +405,7 @@ static VALUE get_def_opts(VALUE self) {
     switch (oj_default_options.escape_mode) {
     case NLEsc: rb_hash_aset(opts, escape_mode_sym, newline_sym); break;
     case JSONEsc: rb_hash_aset(opts, escape_mode_sym, json_sym); break;
+    case SlashEsc: rb_hash_aset(opts, escape_mode_sym, slash_sym); break;
     case XSSEsc: rb_hash_aset(opts, escape_mode_sym, xss_safe_sym); break;
     case ASCIIEsc: rb_hash_aset(opts, escape_mode_sym, ascii_sym); break;
     case JXEsc: rb_hash_aset(opts, escape_mode_sym, unicode_xss_sym); break;
@@ -733,6 +735,8 @@ static int parse_options_cb(VALUE k, VALUE v, VALUE opts) {
             copts->escape_mode = NLEsc;
         } else if (json_sym == v) {
             copts->escape_mode = JSONEsc;
+        } else if (slash_sym == v) {
+            copts->escape_mode = SlashEsc;
         } else if (xss_safe_sym == v) {
             copts->escape_mode = XSSEsc;
         } else if (ascii_sym == v) {
@@ -916,7 +920,7 @@ static int parse_options_cb(VALUE k, VALUE v, VALUE opts) {
 
                 copts->ignore = ALLOC_N(VALUE, cnt + 1);
                 for (i = 0; i < cnt; i++) {
-                    copts->ignore[i] = rb_ary_entry(v, i);
+                    copts->ignore[i] = RARRAY_AREF(v, i);
                 }
                 copts->ignore[i] = Qnil;
             }
@@ -1147,7 +1151,17 @@ static VALUE load_file(int argc, VALUE *argv, VALUE self) {
         }
     }
     path = StringValuePtr(*argv);
-    if (0 == (fd = open(path, O_RDONLY))) {
+#ifdef _WIN32
+    {
+        WCHAR *wide_path;
+        wide_path = rb_w32_mbstr_to_wstr(CP_UTF8, path, -1, NULL);
+        fd = rb_w32_wopen(wide_path, O_RDONLY);
+        free(wide_path);
+    }
+#else
+    fd = open(path, O_RDONLY);
+#endif
+    if (0 == fd) {
         rb_raise(rb_eIOError, "%s", strerror(errno));
     }
     switch (mode) {
@@ -1243,9 +1257,8 @@ static VALUE dump_body(VALUE a) {
 static VALUE dump_ensure(VALUE a) {
     volatile struct dump_arg *arg = (void *)a;
 
-    if (arg->out->allocated) {
-        xfree(arg->out->buf);
-    }
+    oj_out_free(arg->out);
+
     return Qnil;
 }
 
@@ -1257,7 +1270,6 @@ static VALUE dump_ensure(VALUE a) {
  * - *options* [_Hash_] same as default_options
  */
 static VALUE dump(int argc, VALUE *argv, VALUE self) {
-    char            buf[4096];
     struct dump_arg arg;
     struct _out     out;
     struct _options copts = oj_default_options;
@@ -1279,9 +1291,8 @@ static VALUE dump(int argc, VALUE *argv, VALUE self) {
     arg.argc  = argc;
     arg.argv  = argv;
 
-    arg.out->buf       = buf;
-    arg.out->end       = buf + sizeof(buf) - 10;
-    arg.out->allocated = false;
+    oj_out_init(arg.out);
+
     arg.out->omit_nil  = copts.dump_opts.omit_nil;
     arg.out->caller    = CALLER_DUMP;
 
@@ -1313,7 +1324,6 @@ static VALUE dump(int argc, VALUE *argv, VALUE self) {
  * Returns [_String_] the encoded JSON.
  */
 static VALUE to_json(int argc, VALUE *argv, VALUE self) {
-    char            buf[4096];
     struct _out     out;
     struct _options copts = oj_default_options;
     VALUE           rstr;
@@ -1328,9 +1338,9 @@ static VALUE to_json(int argc, VALUE *argv, VALUE self) {
     }
     copts.mode    = CompatMode;
     copts.to_json = Yes;
-    out.buf       = buf;
-    out.end       = buf + sizeof(buf) - 10;
-    out.allocated = false;
+
+    oj_out_init(&out);
+
     out.omit_nil  = copts.dump_opts.omit_nil;
     // For obj.to_json or generate nan is not allowed but if called from dump
     // it is.
@@ -1341,9 +1351,9 @@ static VALUE to_json(int argc, VALUE *argv, VALUE self) {
     }
     rstr = rb_str_new2(out.buf);
     rstr = oj_encode(rstr);
-    if (out.allocated) {
-        xfree(out.buf);
-    }
+
+    oj_out_free(&out);
+
     return rstr;
 }
 
@@ -1703,6 +1713,15 @@ static VALUE protect_require(VALUE x) {
     return Qnil;
 }
 
+extern void print_all_odds(const char *label);
+
+static VALUE
+debug_odd(VALUE self, VALUE label) {
+    print_all_odds(RSTRING_PTR(label));
+    return Qnil;
+}
+
+
 /* Document-module: Oj
  *
  * Optimized JSON (Oj), as the name implies was written to provide speed
@@ -1731,15 +1750,19 @@ static VALUE protect_require(VALUE x) {
  *
  * - *:wab* specifically for WAB data exchange.
  */
-void Init_oj() {
+void Init_oj(void) {
     int err = 0;
 
 #if HAVE_RB_EXT_RACTOR_SAFE
     rb_ext_ractor_safe(true);
 #endif
     Oj = rb_define_module("Oj");
+    rb_gc_register_address(&Oj);
 
     oj_cstack_class = rb_define_class_under(Oj, "CStack", rb_cObject);
+    rb_gc_register_address(&oj_cstack_class);
+
+    rb_undef_alloc_func(oj_cstack_class);
 
     oj_string_writer_init();
     oj_stream_writer_init();
@@ -1748,9 +1771,11 @@ void Init_oj() {
     // On Rubinius the require fails but can be done from a ruby file.
     rb_protect(protect_require, Qnil, &err);
     rb_require("stringio");
-    oj_utf8_encoding = rb_enc_find("UTF-8");
+    oj_utf8_encoding_index = rb_enc_find_index("UTF-8");
+    oj_utf8_encoding = rb_enc_from_index(oj_utf8_encoding_index);
 
     // rb_define_module_function(Oj, "hash_test", hash_test, 0);
+    rb_define_module_function(Oj, "debug_odd", debug_odd, 1);
 
     rb_define_module_function(Oj, "default_options", get_def_opts, 0);
     rb_define_module_function(Oj, "default_options=", set_def_opts, 1);
@@ -1802,7 +1827,6 @@ void Init_oj() {
     oj_hash_set_id           = rb_intern("hash_set");
     oj_hash_start_id         = rb_intern("hash_start");
     oj_iconv_id              = rb_intern("iconv");
-    oj_instance_variables_id = rb_intern("instance_variables");
     oj_json_create_id        = rb_intern("json_create");
     oj_length_id             = rb_intern("length");
     oj_new_id                = rb_intern("new");
@@ -1965,6 +1989,8 @@ void Init_oj() {
     rb_gc_register_address(&ruby_sym);
     sec_prec_sym = ID2SYM(rb_intern("second_precision"));
     rb_gc_register_address(&sec_prec_sym);
+    slash_sym = ID2SYM(rb_intern("slash"));
+    rb_gc_register_address(&slash_sym);
     strict_sym = ID2SYM(rb_intern("strict"));
     rb_gc_register_address(&strict_sym);
     symbol_keys_sym = ID2SYM(rb_intern("symbol_keys"));
@@ -2017,4 +2043,5 @@ void Init_oj() {
     oj_init_doc();
 
     oj_parser_init();
+    oj_scanner_init();
 }
