@@ -345,6 +345,12 @@ class Juice < Minitest::Test
     out = Oj.dump hash
     assert_equal(%{{"key":"I \\u003c3 this"}}, out)
   end
+  def test_escapes_slashes_by_default_when_configured_to_do_so
+    hash = {'key' => "I <3 this </script>"}
+    Oj.default_options = {:escape_mode => :slash}
+    out = Oj.dump hash
+    assert_equal(%{{"key":"I <3 this <\\/script>"}}, out)
+  end
   def test_escapes_entities_when_asked_to
     hash = {'key' => "I <3 this"}
     out = Oj.dump(hash, :escape_mode => :xss_safe)
@@ -528,7 +534,7 @@ class Juice < Minitest::Test
     assert_equal(58, obj.y)
   end
 
-# Stream Deeply Nested
+  # Stream Deeply Nested
   def test_deep_nest_dump
     begin
       a = []
@@ -541,7 +547,7 @@ class Juice < Minitest::Test
     assert(false, "*** expected an exception")
   end
 
-# Stream IO
+  # Stream IO
   def test_io_string
     src = { 'x' => true, 'y' => 58, 'z' => [1, 2, 3]}
     output = StringIO.open("", "w+")
@@ -553,6 +559,9 @@ class Juice < Minitest::Test
   end
 
   def test_io_file
+    # Windows does not support fork
+    return if RbConfig::CONFIG['host_os'] =~ /(mingw|mswin)/
+
     src = { 'x' => true, 'y' => 58, 'z' => [1, 2, 3]}
     filename = File.join(File.dirname(__FILE__), 'open_file_test.json')
     File.open(filename, "w") { |f|
@@ -564,6 +573,28 @@ class Juice < Minitest::Test
     assert_equal(src, obj)
   end
 
+  def test_io_stream
+    IO.pipe do |r, w|
+      if fork
+	r.close
+	#w.nonblock = false
+	a = []
+	10_000.times do |i|
+	  a << i
+	end
+	Oj.to_stream(w, a, indent: 2)
+	w.close
+      else
+	w.close
+	sleep(0.1) # to force a busy
+	cnt = 0
+	r.each_line { cnt += 1 }
+	r.close
+	Process.exit(0)
+      end
+    end
+  end
+
 # comments
   def test_comment_slash
     json = %{{
